				;Zahn-Uhr v1.03
				;
				;- Start-Taste schaltet Betriebsspannung ein
				;  und startet das Programm
				;- 7-Segment-Anzeige zhlt Minuten von 3 auf 0
				;- 6 LEDs zhlen in 10-Sekunden-Schritten
				;- Ende: Tonausgabe auf Piezo-Pieper (4 sek)
				;- Abschaltung der Betriebsspannung
				;
				;Taktgerator im LP-Mode mit 32,768 kHz
				;PortA0: Schalter fr 7-Segment-Anzeige (0=ein)
				;PortA1: Schalter fr Zeiger-LEDs (0=ein)
				;PortA2: Schalter fr Piezo-Pieper (0=ein)
				;PortA3: Betriebsspannungskontrolle (1=ein)
				;PortB0: Start-Taster (Eingang 0=Start)
				;PortB1: Segment-A und LED 1 (0=ein)
				;PortB2: Segment-B und LED 2 (0=ein)
				;PortB3: Segment-C und LED 3 (0=ein)
				;PortB4: Segment-D und LED 4 (0=ein)
				;PortB5: Segment-E und LED 5 (0=ein)
				;PortB6: Segment-F und LED 6 (0=ein)
				;PortB7: Segment-G (0=ein)

	list p=16f84

indf	equ	h'00'		;Index-Daten-Register
timer0	equ	h'01'		;Timer0-Register
status	equ	h'03'		;Status-Register
fsr	equ	h'04'		;Index-Adress-Register
porta	equ	h'05'		;PortA-Register (Bank 0)
portb	equ	h'06'		;PortB-Register (Bank 0)
intcon	equ	h'0b'		;Interrupt-Register
optreg	equ	h'01'		;Option-Register (Bank 1)
trisa	equ	h'05'		;PortA-Richtungsresgister (Bank 1)
trisb	equ	h'06'		;PortB-Richtungsresgister (Bank 1)

w_temp	equ	h'0c'		;Zwischenspeicher fr W bei Int
s_temp	equ	h'0d'		;Zwischenspeicher fr Status bei Int
zykzhl	equ	h'10'		;Zyklus-Zhler, zhlt Timer-Ints (0-31)
sekzhl	equ	h'11'		;Sekunden-Zhler, zhlt Sekunden (0-9)
ledzhl	equ	h'12'		;LED-Zhler, zhlt alle 10 Sekunden (0-5)
minzhl	equ	h'13'		;Minuten-Zhler (0-3)
zykmem	equ	h'14'		;Zyklus-Zhler Zwischenspeicher
intflg	equ	h'15'		;Interrupt-Flag
ledtm1	equ	h'16'		;Zwischenspeicher fr LED-Nummer
ledtm2	equ	h'17'		;Zwischenspeicher fr LED-Zhler

zykini	equ	d'31'		;Startwert fr Zyklus-Zhler
sekini	equ	9		;Startwert fr Sekunden-Zhler
ledini	equ	5		;Startwert fr LED-Zhler
minini	equ	3		;Startwert fr Minuten-Zhler

dsptab	equ	h'20'		;Basis-Adresse fr 7-Segment-Tabelle
dsp00	equ	b'00111111'	;Segmente fr Anzeige '0'
dsp01	equ	b'00000110'	;Segmente fr Anzeige '1'
dsp02	equ	b'01011011'	;Segmente fr Anzeige '2'
dsp03	equ	b'01001111'	;Segmente fr Anzeige '3'

bank0	macro
	bcf	status,5	;Bank0 auswhlen
	endm

bank1	macro
	bsf	status,5	;Bank1 auswhlen
	endm

reset	org	h'000'
	goto	start		;Sprung zum Start

int	org	h'004'
	movwf	w_temp		;W im Zwischenspeicher sichern
	swapf	status,w	;Status-Register swappen und nach W holen
	movwf	s_temp		;Status im Zwischenspeicher sichern

	btfss	intcon,1	;Interrupt durch Starttaste (RB0)?
	goto	inttim		;nein -> Sprung zum Timerinterrupt
	bcf	intcon,1	;sonst RB0-Interrupt-Flag lschen
	bsf	intflg,0	;Interrupt-Flag setzen
	goto	intend		;Ende

inttim	bcf	intcon,2	;Timer Overflow lschen
	decf	zykzhl,f	;Zyklus-Zhler -1
	btfss	zykzhl,7	;Zhlerberlauf? ja -> weiterbearbeiten
	goto	intend		;sonst -> Ende
	movlw	zykini		;Startwert fr Zyklus-Zhler
	movwf	zykzhl		;neu setzen
	decf	sekzhl,f	;Sekunden-Zhler -1
	btfss	sekzhl,7	;Zhlerberlauf? ja -> weiterbearbeiten
	goto	intend		;sonst -> Ende
	movlw	sekini		;Startwert fr Sekunden-Zhler
	movwf	sekzhl		;neu setzen
	decf	ledzhl,f	;LED-Zhler -1
	btfss	ledzhl,7	;Zhlerberlauf? ja -> weiterbearbeiten
	goto	intend		;sonst -> Ende
	movlw	ledini		;Startwert fr LED-Zhler
	movwf	ledzhl		;neu setzen
	decf	minzhl,f	;Minuten-Zhler setzen

intend	swapf	s_temp,w	;gesicherten Status swappen und nach W holen
	movwf	status		;und wieder ins Status-Register schreiben
	swapf	w_temp,f	;Nibbles im gesicherten W tauschen
	swapf	w_temp,w	;Nibbles nochmals tauschen und in W ablegen
	retfie

start	movlw	b'00011111'	;Bit7: Pullups fr PortB aktivieren
	bank1			;Bit6: Interrupt bei fallender Flanke an RB0
	movwf	optreg		;Bit5: Timer0 wird vom System (CLK/4) getaktet
	bank0
	movlw	b'00001111'	;PortA: alle genutzen Ausgnge auf 1 setzen
	movwf	porta		;PortA initialisieren
	movlw	b'11111111'	;PortB: alle Ausgnge auf 1 setzen
	movwf	portb		;PortB initialisieren
	bank1
	clrf	trisa		;PortA: alle Pins sind Ausgnge
	movlw	b'00000001'	;PortB: B0 ist Eingang, Rest sind Ausgnge
	movwf	trisb		;PortB setzen
	bank0
	movlw	dsptab		;Adresse fr 7-Segment-Tabelle
	movwf	fsr		;ins Index-Adress-Register legen
	movlw	dsp00		;Segmentliste fr Ziffer 0 holen
	movwf	indf		;und in die Tabelle schreiben
	incf	fsr,f		;Zeiger auf nchsten Tabellenwert
	movlw	dsp01		;Segmentliste fr Ziffer 1 holen
	movwf	indf		;und in die Tabelle schreiben
	incf	fsr,f		;Zeiger auf nchsten Tabellenwert
	movlw	dsp02		;Segmentliste fr Ziffer 2 holen
	movwf	indf		;und in die Tabelle schreiben
	incf	fsr,f		;Zeiger auf nchsten Tabellenwert
	movlw	dsp03		;Segmentliste fr Ziffer 3 holen
	movwf	indf		;und in die Tabelle schreiben
	clrf	zykmem		;Zyklus-Zhler Zwischenspeicher lschen
	clrf	intflg		;Interrupt-Flag lschen

	clrf	timer0		;Timer initialisieren
	movlw	b'10110000'	;RB0/Timer0/alle Interrupts aktivieren
	movwf	intcon

	movlw	zykini		;Startwert fr Zyklus-Zhler
	movwf	zykzhl		;Zyklus-Zhler setzen
	movlw	sekini		;Startwert fr Sekunden-Zhler
	movwf	sekzhl		;Sekunden-Zhler setzen
	movlw	ledini		;Startwert fr LED-Zhler
	movwf	ledzhl		;LED-Zhler setzen
	movlw	minini		;Startwert fr Minuten-Zhler
	movwf	minzhl		;Minuten-Zhler setzen
	clrf	zykmem		;Zyklus-Zhler Zwischenspeicher lschen
	clrf	intflg		;Interrupt-Flag lschen

hprogr	btfsc	intflg,0	;Interrupt-Flag gesetzt?
	goto	rstart		;ja -> Neustart des Systems

	movf	zykzhl,w	;sonst Zyklus-Zhler nach W holen und
	subwf	zykmem,w	;mit dem Zyklus-Zwischenspeicher vergleichen
	btfsc	status,2	;hat sich der Zhler verndert (Z=0)?
	goto	hprogr		;nein -> Schleife
	movf	zykzhl,w	;sonst Zyklus-Zhler nach W holen
	movwf	zykmem		;und zwischenspeichern
	movf	minzhl,w	;Minuten-Zhler nach W holen
	btfsc	status,2	;Zhler=0?
	goto	hprend		;ja -> zum Ende-Zyklus springen
	movf	zykzhl,w	;sonst Zyklus-Zhler nach W holen
	andlw	b'00001111'	;die unteren 4 Bits ausfiltern (= 1/2s, 1:15)
	sublw	b'00001111'	;bei '1111' 7-Segment-Anzeige einschalten
	btfsc	status,2	;Zyklus '1111'?
	goto	dspein		;ja -> 7-Segment-Anzeige einschalten
	movf	zykzhl,w	;sonst Zyklus-Zhler nach W holen
	andlw	b'00001111'	;die unteren 3 Bits ausfiltern (= 1/2s, 1:15)
	sublw	b'00000111'	;bei '0111' LEDs einschalten
	btfsc	status,2	;Zyklus '0111'?
	goto	ledein		;ja -> LEDs einschalten
	movlw	b'11111111'	;sonst alle 7-Segmente und LEDs
	movwf	portb		;auschalten
	movlw	b'00001111'	;und Treiber
	movwf	porta		;ausschalten
	goto	hprogr

dspein	movf	minzhl,w	;Minutenzhler holen
	andlw	b'00001111'	;unteres Nibble ausblenden
	addlw	dsptab		;Basisadresse fr 7-Segment-Tabelle addieren
	movwf	fsr		;ins Index-Adress-Register schreiben
	bsf	status,0	;Carry-Flag setzen
	rlf	indf,w		;Segmentbelegung auf B1-B7 verschieben
	xorlw	b'11111110'	;Segmentbelegung invertieren (L-aktiv)
	movwf	portb		;und ausgeben
	bcf	porta,0		;Treiber fr 7-Segment-Anzeige aktivieren
	goto	hprogr

ledein	movlw	b'01111111'	;Startwert fr LED-Nummer
	movwf	ledtm1		;zwischenspeichern
	incf	ledzhl,w	;LED-Zhler um 1 erhhen und nach W holen
	movwf	ledtm2		;und zwischenspeichern
	bsf	status,0	;Carry-Flag setzen
ledloo	rrf	ledtm1,f	;nchste LED aktivieren
	decfsz	ledtm2,f	;LED-Zhler um 1 vermindern
	goto	ledloo		;0 erreicht? nein -> nchste LED aktivieren
	movf	ledtm1,w	;sonst LED-Nummer holen
	movwf	portb		;und an Port ausgeben
	bcf	porta,1		;Treiber fr LEDs aktivieren
	goto	hprogr

hprend	movf	sekzhl,w	;LED-Zhler nach W holen
	sublw	5		;nach 4 sek (9-5) beenden
	btfsc	status,2	;4 sek Zyklus beendet?
	goto	ende		;ja -> Programm anhalten
	movlw	b'10000001'	;sonst Segmente a-f (=LED 1-6) aktivieren
	movwf	portb		;und ausgeben
	movf	zykzhl,w	;Zyklus-Zhler nach W holen
	andlw	b'00001111'	;die unteren 4 Bits ausfiltern (= 1/2s, 1:15)
	sublw	b'00001111'	;bei '1111' 7-Segment-Anzeige einschalten
	btfss	status,2	;Zyklus '1111'?
	goto	eledte		;nein -> weiter
	bcf	porta,0		;sonst 7-Segment-Anzeige einschalten
	goto	tonaus		;Sprung zur Tonausgabe
eledte	movf	zykzhl,w	;Zyklus-Zhler nach W holen
	andlw	b'00001111'	;die unteren 4 Bits ausfiltern (= 1/2s, 1:15)
	sublw	b'00000111'	;bei '0111' LEDs einschalten
	btfss	status,2	;Zyklus '0111'?
	goto	eledau		;nein -> 7-Segment-Anzeige und LEDs ausschalten
	bcf	porta,1		;sonst LEDs einschalten
	goto	tonaus		;Sprung zur Tonausgabe
eledau	bsf	porta,0		;7-Segment-Anzeige ausschalten
	bsf	porta,1		;LEDs ausschalten
tonaus	movf	zykzhl,w	;sonst Zyklus-Zhler nach W holen
	andlw	b'00001100'	;Bits 4-3 ausfiltern (= 1/2s, 4:12)
	sublw	b'00001100'	;bei '11xx' Ton einschalten
	btfss	status,2	;Zyklus '11xx'?
	goto	etonau		;nein -> Ton ausschalten
	bcf	porta,2		;sonst Ton einschalten
	goto	hprogr		;Schleife
etonau	bsf	porta,2		;Ton Ausschalten
	goto	hprogr		;Schleife

rstart	bcf	intcon,5	;Timer-Interrupt deaktivieren
	movlw	b'11111111'	;alle Segmente und LEDs
	movwf	portb		;ausschalten
	movlw	b'00001111'	;alle Treiber
	movwf	porta		;ausschalten
	goto	start		;Programm neu starten

ende	bcf	intcon,5	;Timer-Interrupt deaktivieren
	movlw	b'11111111'	;alle Segmente und LEDs
	movwf	portb		;ausschalten
	movlw	b'00000111'	;alle Treiber und Stromversorgung
	movwf	porta		;ausschalten
endelo	goto	endelo		;Endlosschleife

	org	h'2100'		;EEPROM-Bereich
	de	"ZahnUhr v1.03 (25.08.2000)"

	__config b'11111111110000'	;Bit13-4: keine Protection
					;Bit3: Power-up Timer ein
					;Bit2: Watchdog Timer aus
					;Bit1-0: LP-Oszillator
	end

